# pylint: disable=invalid-name

'''
General helper functions for pyVmomi
Author: Philipp Schmitt <philipp.schmitt@post.lu>
'''

from __future__ import print_function
from pyVmomi import vim
import re


def get_object_by_name(content, object_type, name, regex=False):
    '''
    Get the vsphere object associated with a given text name
    Source: https://github.com/rreubenur/vmware-pyvmomi-examples/blob/master/create_template.py
    '''
    container = content.viewManager.CreateContainerView(
        content.rootFolder, [object_type], True
    )
    for c in container.view:
        if regex:
            if re.match(name, c.name):
                return c
        elif c.name == name:
            return c


def get_vm_by_name(content, name, regex=False):
    '''
    Get a VM by its name
    '''
    return get_object_by_name(content, vim.VirtualMachine, name, regex)


def get_vm_by_ip(content, ip, multiple=False):
    '''
    Get a VM by its IP address
    '''
    matches = []
    for vm in get_all_vms(content):
        for ipconfig in [x.ipConfig.ipAddress for x in vm.guest.net \
                if hasattr(x.ipConfig, 'ipAddress')]:
            for c in ipconfig:
                if c.ipAddress == ip:
                    matches.append(vm)
                    if not multiple:
                        break
    if len(matches) == 0:
        return
    elif len(matches) == 1:
        return matches[0]
    else:
        return matches


def get_all(content, container, object_type):
    '''
    Get all items of a certain type
    Example: get_all(content, vim.Datastore) return all datastore objects
    '''
    obj_list = list()
    view_manager = content.viewManager
    object_view = view_manager.CreateContainerView(
        container, [object_type], True
    )
    for obj in object_view.view:
        if isinstance(obj, object_type):
            obj_list.append(obj)
    object_view.Destroy()
    return obj_list


def get_all_vms(content):
    '''
    Get all VMs managed by a vCenter
    '''
    return get_all(content, content.rootFolder, vim.VirtualMachine)


def get_all_datastores(content):
    '''
    Get all datastore on a vCenter
    '''
    return get_all(content, content.rootFolder, vim.Datastore)


def get_all_clusters(content):
    '''
    Get all hosts on a vCenter
    '''
    return get_all(content, content.rootFolder, vim.ClusterComputeResource)


def get_all_hosts(content):
    '''
    Get all clusters on a vCenter
    '''
    return get_all(content, content.rootFolder, vim.HostSystem)


def get_hosts_in_datacenter(content, datacenter):
    '''
    Get all hosts belonging to a given datacenter
    '''
    return get_all(content, datacenter, vim.HostSystem)


def get_vms_in_datacenter(content, datacenter):
    '''
    Get all vms belonging to a given datacenter
    '''
    return get_all(content, datacenter, vim.VirtualMachine)


def get_datacenter(content, obj):
    '''
    Get the datacenter to whom an object belongs
    '''
    datacenters = content.rootFolder.childEntity
    for d in datacenters:
        dch = get_all(content, d, type(obj))
        if dch is not None and obj in dch:
            return d


def get_all_dvswitches(content):
    '''
    Get all the distributed switches
    '''
    return get_all(content, content.rootFolder, vim.DistributedVirtualSwitch)


def get_all_dport_groups(content):
    '''
    Get all the distributed port groups
    '''
    return get_all(
        content, content.rootFolder, vim.dvs.DistributedVirtualPortgroup
    )


def get_all_vswitches(content):
    '''
    Get all the virtual switches
    '''
    vswitches = []
    hosts = get_all_hosts(content)
    for h in hosts:
        for s in h.config.network.vswitch:
            vswitches.append(s)
    return vswitches


def print_vm_info(vm):
    '''
    Print information for a particular virtual machine
    '''
    summary = vm.summary
    print('Name  : ', summary.config.name)
    print('Path  : ', summary.config.vmPathName)
    print('Guest : ', summary.config.guestFullName)
    annotation = summary.config.annotation
    if annotation is not None and annotation != '':
        print('Annotation : ', annotation)
    print('State : ', summary.runtime.powerState)
    if summary.guest is not None:
        ip = summary.guest.ipAddress
        if ip is not None and ip != '':
            print('IP    : ', ip)
    if summary.runtime.question is not None:
        print('Question : ', summary.runtime.question.text)
    print('')



from setuptools import find_packages, setup


setup(
    name="shortmomi",
    version="1.7.1",
    license="GPL3",
    description="Bundle of useful functions and shortcuts for pyVmomi",
    author="Philipp Schmitt",
    author_email="philipp@schmitt.co",
    url="https://github.com/pschmitt/shortmomi",
    packages=find_packages(),
    install_requires=["pyvmomi"],
)



#!/usr/bin/python

'''
Connect
Author: Philipp Schmitt <philipp.schmitt@post.lu>
'''


from __future__ import print_function
from pyVim.connect import Disconnect
from pyVim.connect import SmartConnect
from pyVmomi import vmodl
import atexit
import requests
import ssl
import sys
import traceback


'''
Custom exception that is be thrown when the connection to a vCenter failed
'''
class ConnectionError(RuntimeError):
    pass


def connect(host, username, password, port=443, verify=False, debug=False):
    '''
    Connect to a vCenter via the API
    :param host: Hostname or IP of the vCenter
    :type host: str or unicode
    :param username: Username
    :type user: str or unicode
    :param password: Password
    :type user: str or unicode
    :param port: Port on which the vCenter API is running (default: 443)
    :type port: int
    :param verify: Whether to verify SSL certs upon connection (default: False)
    :type verify: bool
    :param debug: Debug option (default: False)
    :type debug: bool
    :return: Content
    :rtype: vim.ServiceInstanceContent
    '''
    context = ssl.SSLContext(ssl.PROTOCOL_TLSv1_2)
    if not verify:
        # Disable warnings about unsigned certificates
        context.verify_mode = ssl.CERT_NONE
        requests.packages.urllib3.disable_warnings()

    try:
        si = SmartConnect(
            host=host,
            user=username,
            pwd=password,
            port=port,
            sslContext=context
        )
        # Register auto disconnect
        atexit.register(Disconnect, si)
        # Return content
        return si.RetrieveContent()
    except IOError as e:
        print('I/O error({0}): {1}'.format(e.errno, e.strerror))
    except vmodl.MethodFault as e:
        print('Connection could not be established', file=sys.stderr)
        raise ConnectionError('Connection could not be established')
        print('Caught vmodl fault: ', e.msg, file=sys.stderr)
        if debug:
            traceback.print_exc()
    except Exception as e:
        print('Caught exception:', str(e), file=sys.stderr)
        if debug:
            traceback.print_exc()
